<?php
/**
* 2007-2018 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2018 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

class PayBySquareRedirectModuleFrontController extends ModuleFrontController
{
    public function __construct()
    {
        $this->display_column_right = false;
        $this->display_column_left = false;

        parent::__construct();
    }
    /**
     * Do whatever you have to before redirecting the customer on the website of your payment processor.
     */
    public function postProcess()
    {
        /**
         * Oops, an error occured.
         */
        if (Tools::getValue('action') == 'error') {
            return $this->displayError('An error occurred while trying to redirect the customer');
        } else {
            $module = Module::getInstanceByName('paybysquare');
            $context = Context::getContext();
            $cart = $context->cart;

            $pbsm = new PayBySquareMails();

            $cart_addressess = $pbsm->parseAddressDetails($cart->id, 'cart');

            $order_details = $pbsm->parseOrderDetails($cart->id, 'cart');

            $this->context->smarty->assign(array(

                'navigationPipe' => '>',
                
                'cart_addresses' => $cart_addressess,
                'order_details' => $order_details,

                'nbProducts' => $cart->nbProducts(),
                'cust_currency' => $cart->id_currency,
                'currencies' => $module->getCurrency((int)$cart->id_currency),
                'total' => $cart->getOrderTotal(true, Cart::BOTH),
                'this_path' => $module->getPathUri(),
                'this_path_bw' => $module->getPathUri(),
                'this_path_ssl' => Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$module->name.'/',
            
                'cart_id' => $cart->id,
                'secure_key' => $context->customer->secure_key,
            ));
            if (version_compare(Tools::substr(_PS_VERSION_, 0, 3), '1.6', '=')) {
                return $this->setTemplate('redirect.tpl');
            } else {
                return $this->setTemplate('module:PayBySquare/views/templates/front/redirect.tpl');
            }
        }
    }

    protected function displayError($message, $description = false)
    {
        /**
         * Create the breadcrumb for your ModuleFrontController.
         */
        $this->context->smarty->assign('path', '
			<a href="'.$this->context->link->getPageLink('order', null, null, 'step=3').'">'.$this->module->l('Payment').'</a>
			<span class="navigation-pipe">&gt;</span>'.$this->module->l('Error'));

        /**
         * Set error message and description for the template.
         */
        array_push($this->errors, $this->module->l($message), $description);

        return $this->setTemplate('error.tpl');
    }
}
